using System;
using System.Collections.Generic;
using System.Text;

using NUnit.Framework;
using NMock2;

namespace MocksArentStubs
{
    [TestFixture]
    public class OrderInteractionTester 
    {
        private static String TALISKER = "Talisker";

        [Test]
        public void testFillingRemovesInventoryIfInStock()
        {
            //setup - data
            Order order = new Order(TALISKER, 50);
            
            Mockery mocks = new Mockery();
            IWarehouse warehouseMock = mocks.NewMock<IWarehouse>();

            //setup - expectations
            using (mocks.Ordered) 
            {
                Expect.Once.On(warehouseMock)
                    .Method("hasInventory")
                    .With(TALISKER,50)
                    .Will(Return.Value(true));
                Expect.Once.On(warehouseMock)
                    .Method("remove")
                    .With(TALISKER, 50);
            }
            //exercise
            order.fill(warehouseMock);

            //verify
            mocks.VerifyAllExpectationsHaveBeenMet();
        }

        [Test]
        public void testFillingDoesNotRemoveIfNotEnoughInStock()
        {
            Order order = new Order(TALISKER, 50);

            Mockery mocks = new Mockery();
            IWarehouse warehouseMock = mocks.NewMock<IWarehouse>();

            Expect.Once.On(warehouseMock)
                .Method("hasInventory")
                .WithAnyArguments()
                .Will(Return.Value(false));

            order.fill(warehouseMock);

            mocks.VerifyAllExpectationsHaveBeenMet();
        }
    }

   
}
