using System;
using System.Collections ;
using System.Collections.Generic;
using System.Text;

using NUnit.Framework;



namespace MocksArentStubs
{

    public interface IWarehouse
    {
        void add(string itemIdentification, int orderAmount);
        void remove(string itemIdentification, int orderAmount);
        int getInventory(string itemIdentification);
        Boolean hasInventory(string itemIdentification, int orderAmount);
    }

    public class Warehouse : IWarehouse
    {
        private Hashtable AvailableItems = new Hashtable();
 
        public void add(string itemIdentification, int orderAmount)
        {
            AvailableItems.Add(itemIdentification, orderAmount);
        }

        public void remove(string itemIdentification, int orderAmount)
        {
           AvailableItems[itemIdentification] = (int) AvailableItems[itemIdentification] - orderAmount;
        }

        public Boolean hasInventory(string itemIdentification, int orderAmount)
        {
            return (int) AvailableItems[itemIdentification] >= orderAmount ;
        }

        public int getInventory(string itemIdentification)
        {
            return (int) AvailableItems[itemIdentification]; ;
        }
    }

    public class Order
    {
        private string Identification;
        private int Amount;
        private Boolean Filled;

        public Order(string itemIdentification, int orderAmount)
        {
            Identification = itemIdentification;
            Amount = orderAmount;
            Filled = false;
        }

        public void fill(IWarehouse warehouse)
        {
            if (warehouse.hasInventory(Identification,Amount))
            {
                warehouse.remove(Identification, Amount);
                Filled = true;
            }  
        }
        public Boolean isFilled()
        {
            return Filled;
        }
    }


    [TestFixture]
    public class TestsSelonApprocheTraditionnelle
    {
        private static String TALISKER = "Talisker";
        private static String HIGHLAND_PARK = "Highland Park";
        private Warehouse warehouse;

        [SetUp]
        public void setUp()
        {
            warehouse = new Warehouse() ;
            warehouse.add(TALISKER, 50);
            warehouse.add(HIGHLAND_PARK, 25);
        }

        [Test]
        public void testOrderIsFilledIfEnoughInWarehouse()
        {
            Order order = new Order(TALISKER, 50);
            Console.WriteLine("before order filled");
            order.fill(warehouse);
            Console.WriteLine("order filled");

            Assert.IsTrue(order.isFilled());
            Assert.AreEqual(0, warehouse.getInventory(TALISKER));
        }

        [Test]
        public void testOrderDoesNotRemoveIfNotEnough()
        {
            Order order = new Order(TALISKER, 51);
            order.fill(warehouse);
            Assert.IsFalse(order.isFilled());
            Assert.AreEqual(50, warehouse.getInventory(TALISKER));
        }

    }

}
